#!/usr/bin/env node
import { existsSync } from 'fs';
import path from 'path';
import process from 'process';
import { URL } from 'url';
import browserSync from 'browser-sync';
import chalk from 'chalk';
import chokidar from 'chokidar';
import commandLineArgs from 'command-line-args';
import merge from 'deepmerge';
import getPort, { portNumbers } from 'get-port';
import { AfterTransformPluginError, TransformPluginError, TemplateRenderError } from './utilities/errors.js';
import { isMarkdownFile } from './utilities/file.js';
import { clean, copyAssets, copyTheme, publish } from './utilities/publish.js';
const bs = browserSync.create();
const currentDir = new URL('.', import.meta.url).pathname;
let targetDirectory = process.cwd();
let config;
export * from './utilities/types';
export const defaultConfig = {
    assetFolderName: 'assets',
    cleanOnPublish: true,
    copyAssets: ['assets/**/*'],
    data: {},
    dev: false,
    helpers: [],
    inputDir: '.',
    outputDir: '_site',
    partials: [],
    plugins: [],
    themeDir: path.join(currentDir, '../themes/default'),
    themeFolderName: 'theme'
};
let options;
try {
    options = {
        dir: targetDirectory,
        serve: false,
        watch: false,
        ...commandLineArgs([
            { name: 'dir', alias: 'd', type: String },
            { name: 'serve', alias: 's', type: Boolean },
            { name: 'watch', alias: 'w', type: Boolean }
        ])
    };
}
catch (err) {
    console.error(chalk.red(err.message));
    process.exit(1);
}
if (options.serve) {
    options.watch = true;
}
if (options.dir) {
    try {
        targetDirectory = path.resolve(options.dir);
        process.chdir(targetDirectory);
    }
    catch (err) {
        console.error(chalk.red(`Invalid target directory: "${options.dir}"`));
        process.exit(2);
    }
}
const userConfigFilename = path.join(targetDirectory, 'doxicity.config.js');
if (existsSync(userConfigFilename)) {
    const userConfig = (await import(userConfigFilename)).default;
    config = merge(defaultConfig, userConfig);
}
else {
    config = { ...defaultConfig };
    console.warn(chalk.yellow(`No config filed found at "${userConfigFilename}". Using default options!`));
}
async function publishPages() {
    try {
        const result = await publish(config);
        const numPages = result.publishedPages.length;
        const term = numPages === 1 ? 'one page' : `${numPages} pages`;
        console.log(chalk.green(`Doxicity successfully published ${term} to "${config.outputDir}"`));
    }
    catch (err) {
        if (err instanceof AfterTransformPluginError ||
            err instanceof TemplateRenderError ||
            err instanceof TransformPluginError) {
            console.error(chalk.red(`${err.page.inputFile}: ${err.message}`));
        }
        else {
            console.error(chalk.red(err.message ?? err));
        }
    }
}
if (config.cleanOnPublish) {
    try {
        await clean(config);
    }
    catch (err) {
        console.error(chalk.yellow(`Unable to clean the output directory before publishing: ${err.message}`));
    }
}
try {
    await Promise.all([copyAssets(config), copyTheme(config)]);
    await publishPages();
}
catch (err) {
    console.error(chalk.red(err.message));
}
if (options.watch) {
    console.log(`Watching for changes in: ${targetDirectory}`);
    const pathsToWatch = [path.join(targetDirectory, '**/*')];
    if (config.dev) {
        pathsToWatch.push(path.join(config.themeDir));
    }
    const watcher = chokidar.watch(pathsToWatch, {
        persistent: true,
        ignored: [path.resolve(config.outputDir)],
        ignoreInitial: true
    });
    watcher
        .on('add', async (filename) => {
        if (isMarkdownFile(filename)) {
            console.log(`New page created: "${filename}"`);
            await publishPages();
        }
        else {
            console.log(`New file created: "${filename}"`);
            await Promise.all([copyAssets(config), copyTheme(config)]);
        }
        if (options.serve) {
            bs.reload();
        }
    })
        .on('change', async (filename) => {
        console.log('change');
        if (isMarkdownFile(filename)) {
            console.log(`Page changed: "${filename}"`);
            await publishPages();
        }
        else {
            console.log(`File changed: "${filename}"`);
            await Promise.all([copyAssets(config), copyTheme(config)]);
            await publishPages();
        }
        if (options.serve) {
            bs.reload();
        }
    })
        .on('unlink', async (filename) => {
        console.log(`Page removed: "${filename}"`);
        try {
            await clean(config);
        }
        catch (err) {
            console.error(chalk.yellow(`Unable to clean the output directory before publishing: ${err.message}`));
        }
        try {
            await Promise.all([copyAssets(config), copyTheme(config)]);
            await publishPages();
        }
        catch (err) {
            console.error(chalk.red(err.message));
        }
        if (options.serve) {
            bs.reload();
        }
    });
}
if (options.serve) {
    const port = await getPort({
        port: portNumbers(4000, 4999)
    });
    bs.init({
        open: true,
        startPath: '/',
        port,
        logLevel: 'silent',
        logPrefix: '[doxicity]',
        logFileChanges: false,
        notify: false,
        ghostMode: false,
        server: {
            baseDir: config.outputDir
        }
    }, () => {
        const url = `http://localhost:${port}`;
        console.log(chalk.cyan(`Launched the Doxicity dev server at ${url} 📚\n`));
    });
}
