import fs from 'fs/promises';
import path from 'path';
import lunr from 'lunr';
import { getRelativeUrl } from '../../utilities/file.js';
const map = [];
function stripWhitespace(string) {
    return string.replace(/\s+/g, ' ');
}
export default function (options) {
    const pagesToIndex = [];
    let hasClientBeenCopied = false;
    options = {
        ignore: () => false,
        getTitle: doc => doc.querySelector('h1')?.textContent ?? doc.querySelector('title')?.textContent ?? '',
        getDescription: doc => doc.querySelector('meta[name="description"]')?.getAttribute('content') ?? '',
        getHeadings: doc => [...doc.querySelectorAll('h2, h3, h4')].map(heading => heading.textContent ?? ''),
        getContent: doc => doc.querySelector('body')?.textContent ?? '',
        searchDirName: 'search',
        ...options
    };
    return {
        transform: async (doc, page, config) => {
            const searchDir = path.join(config.outputDir, config.assetFolderName, options.searchDirName);
            const scriptFilename = path.join(searchDir, 'client.js');
            const stylesFilename = path.join(searchDir, 'client.css');
            if (!hasClientBeenCopied) {
                await fs.mkdir(path.dirname(scriptFilename), { recursive: true });
                await Promise.all([
                    fs.copyFile('../src/plugins/search/client.js', scriptFilename),
                    fs.copyFile('../src/plugins/search/client.css', stylesFilename)
                ]);
                hasClientBeenCopied = true;
            }
            const script = doc.createElement('script');
            script.src = getRelativeUrl(config, scriptFilename);
            script.type = 'module';
            doc.body.append(script);
            const link = doc.createElement('link');
            link.rel = 'stylesheet';
            link.href = getRelativeUrl(config, stylesFilename);
            doc.head.append(link);
            const title = stripWhitespace(options.getTitle(doc));
            const description = stripWhitespace(options.getDescription(doc));
            const headings = options.getHeadings(doc).map(stripWhitespace);
            const content = stripWhitespace(options.getContent(doc));
            const url = getRelativeUrl(config, page.outputFile);
            pagesToIndex.push({ page, title, description, headings, content, url });
            return doc;
        },
        afterAll: (_pages, config) => {
            const searchIndexFilename = path.join(config.outputDir, config.assetFolderName, path.join(options.searchDirName, 'search.json'));
            const searchIndex = lunr(async function () {
                let index = 0;
                this.ref('id');
                this.field('t', { boost: 10 });
                this.field('h', { boost: 5 });
                this.field('c');
                for (const page of pagesToIndex) {
                    if (options.ignore(page.page))
                        continue;
                    this.add({ id: index, t: page.title, h: page.headings, c: page.content });
                    map[index] = { title: page.title, description: page.description, url: page.url };
                    index++;
                }
                await fs.mkdir(path.dirname(searchIndexFilename), { recursive: true });
                await fs.writeFile(searchIndexFilename, JSON.stringify({ searchIndex, map }), 'utf8');
            });
        }
    };
}
