import fs from 'fs/promises';
import path from 'path';
import cpy from 'cpy';
import merge from 'deepmerge';
import { globby } from 'globby';
import { JSDOM } from 'jsdom';
import { registerAssetHelper, registerThemeHelper } from '../helpers/built-ins/paths.js';
import { parse as parseMarkdown } from '../utilities/markdown.js';
import { registerHelper, registerPartial, render } from '../utilities/template.js';
import { CleanError, ConfigError, TransformPluginError, AfterTransformPluginError, AfterAllPluginError, TemplateRenderError } from './errors.js';
import { getHtmlFilename } from './file.js';
function checkConfig(config) {
    if (!config.inputDir) {
        throw new ConfigError('Invalid inputDir in your config. You need to tell Doxicity which files to process!');
    }
    if (!config.outputDir) {
        throw new ConfigError('Invalid outputDir in your config. You need to tell Doxicity where to publish your site!');
    }
    if (!config.assetFolderName || config.assetFolderName.includes('/') || config.assetFolderName.includes('\\')) {
        throw new ConfigError(`Invalid assetFolderName in your config: "${config.assetFolderName}". This should be a folder name, not a path.`);
    }
    if (!config.themeFolderName || config.themeFolderName.includes('/') || config.themeFolderName.includes('\\')) {
        throw new ConfigError(`Invalid themeFolderName in your config: "${config.themeFolderName}". This should be a folder name, not a path.`);
    }
}
export async function clean(config) {
    checkConfig(config);
    if (config.cleanOnPublish) {
        try {
            await fs.rm(config.outputDir, { recursive: true, force: true });
        }
        catch (err) {
            throw new CleanError(`Unable to clean the output directory: "${config.outputDir}"`);
        }
    }
}
export async function publish(config) {
    const publishedPages = [];
    checkConfig(config);
    registerAssetHelper(config);
    registerThemeHelper(config);
    config.helpers.forEach(helper => registerHelper(helper.name, helper.callback));
    config.partials.forEach(partial => registerPartial(partial.name, partial.template));
    const sourceFiles = await globby(path.join(config.inputDir, '**/*.md'));
    for (const file of sourceFiles) {
        const parsed = await parseMarkdown(file);
        const templateName = typeof parsed.frontMatter.template === 'string' ? parsed.frontMatter.template : 'default';
        const templateData = merge(config.data, parsed.frontMatter);
        const outFile = getHtmlFilename(config, file);
        let html = '';
        templateData.content = parsed.content;
        const page = {
            inputFile: file,
            outputFile: outFile,
            data: templateData
        };
        try {
            html = await render(page, templateName, templateData, config);
        }
        catch (err) {
            throw new TemplateRenderError(page, err.message);
        }
        const doc = new JSDOM(html).window.document;
        for (const plugin of config.plugins) {
            if (plugin.transform) {
                try {
                    await plugin.transform(doc, page, config);
                }
                catch (err) {
                    throw new TransformPluginError(page, err.message);
                }
            }
        }
        html = doc.documentElement.outerHTML;
        for (const plugin of config.plugins) {
            if (plugin.afterTransform) {
                try {
                    html = await plugin.afterTransform(html, page, config);
                }
                catch (err) {
                    throw new AfterTransformPluginError(page, err.message);
                }
            }
        }
        await fs.mkdir(path.dirname(outFile), { recursive: true });
        await fs.writeFile(outFile, html, 'utf8');
        publishedPages.push(page);
    }
    for (const plugin of config.plugins) {
        if (plugin.afterAll) {
            try {
                await plugin.afterAll(publishedPages, config);
            }
            catch (err) {
                throw new AfterAllPluginError(publishedPages, err.message);
            }
        }
    }
    return { publishedPages };
}
export async function copyAssets(config) {
    const assetPath = path.join(config.outputDir, config.assetFolderName);
    const filesToCopy = config.copyAssets.map(glob => path.resolve(config.inputDir, glob));
    checkConfig(config);
    await fs.mkdir(assetPath, { recursive: true });
    await cpy(filesToCopy, assetPath);
}
export async function copyTheme(config) {
    const themeDir = path.join(config.outputDir, config.themeFolderName);
    checkConfig(config);
    await fs.mkdir(themeDir, { recursive: true });
    await cpy([path.join(config.themeDir, '**/*'), '!**/*.hbs'], themeDir);
}
